//
//  KBEntity.h
//  KBAPISupport
//
//  Created by Kirill byss Bystrov on 28.11.12.
//  Copyright (c) 2012 Kirill byss Bystrov. All rights reserved.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//

#import <Foundation/Foundation.h>

#import "KBAPISupport-config.h"

#if KBAPISUPPORT_XML
@class GDataXMLElement;
#endif

/** This protocol defines the interface of autoconstructed response object.
  * Depending on KBAPISupport config you must override entityFromJSON:,
  * entityFromJSON: or both. For non-ARC environments you may consider
  * using KBAutoEntity.
  */
@protocol KBEntity <NSObject>
@required

/** ------------------------------
  * @name Autoconstructing objects
  * ------------------------------
  */

#if KBAPISUPPORT_JSON
/** Autoconstruct object from JSON response.
  *
  * This method is only available if KBAPISupport is compiled with JSON support.
  *
  * @param JSON Source JSON object.
  * @return Newly constructed object or nil if error occured.
  */
+ (instancetype) entityFromJSON: (id) JSON;
#endif

#if KBAPISUPPORT_XML
/** Autoconstruct object from XML response.
 *
 * This method is only available if KBAPISupport is compiled with XML support.
 *
 * @param XML Source XML element.
 * @return Newly constructed object or nil if error occured.
 */
+ (instancetype) entityFromXML: (GDataXMLElement *) XML;
#endif

@end
